/*
 * @Author: luzc
 * @Date: 2019-10-01 17:01:38
 * @LastEditTime: 2019-10-01 17:06:03
 * @LastEditors: luzc
 * @Description: build 打包cli
 */
const fse = require('fs-extra');
const webpack = require('webpack');

const log = require('../utils/log');
const checkDepsInstalled = require('../utils/checkDepsInstalled');

/**
 * 构建项目：cli 调用
 * 因为 build 方法有可能被直接调用，因此需要保证自身功能的完整性
 * @param {Object} context 命令行参数
 */
module.exports = async function (context) {
  const { applyHook, commandArgs, rootDir, webpackConfig } = context;
  log.verbose('build cliOptions', commandArgs);

  await applyHook('beforeBuild');

  const installedDeps = checkDepsInstalled(rootDir);
  if (!installedDeps) {
    return Promise.reject(new Error('项目依赖未安装，请先安装依赖。'));
  }

  // 清空输出目录
  fse.emptyDirSync(webpackConfig.output.path);

  return new Promise((resolve, reject) => {
    webpack(webpackConfig, (error, stats) => {
      if (error) {
        return reject(error);
      }
      console.log(
        stats.toString({
          colors: true,
          chunks: false,
          children: false,
          modules: false,
          chunkModules: false,
        })
      );
      if (stats.hasErrors()) {
        return reject(new Error('webpack compiled failed.'));
      }
      log.info('ZEN build finished');
      applyHook('afterBuild', stats);
      resolve();
    });
  });
};
