/*
 * @Author: luzc
 * @Date:  2019/10/02
 * @Description: babel 配置
 */

const resolvePlugin = plugins => plugins.filter(Boolean).map(plugin => {
  if (Array.isArray(plugin)) {
    const [pluginName, ...args] = plugin;
    return [require.resolve(pluginName), ...args];
  }
  return require.resolve(plugin);
});

module.exports = (zenConfig = {}) => {
  const plugins = [
    // Stage 0
    '@babel/plugin-proposal-function-bind',

    // Stage 1
    '@babel/plugin-proposal-export-default-from',
    '@babel/plugin-proposal-logical-assignment-operators',
    ['@babel/plugin-proposal-optional-chaining', { loose: false }],
    ['@babel/plugin-proposal-pipeline-operator', { proposal: 'minimal' }],
    ['@babel/plugin-proposal-nullish-coalescing-operator', { loose: false }],
    '@babel/plugin-proposal-do-expressions',

    // Stage 2
    ['@babel/plugin-proposal-decorators', { legacy: true }],
    '@babel/plugin-proposal-function-sent',
    '@babel/plugin-proposal-export-namespace-from',
    '@babel/plugin-proposal-numeric-separator',
    '@babel/plugin-proposal-throw-expressions',

    // Stage 3
    '@babel/plugin-syntax-dynamic-import',
    '@babel/plugin-syntax-import-meta',
    ['@babel/plugin-proposal-class-properties', { loose: true }],
    '@babel/plugin-proposal-json-strings',
  ];

  return {
    presets: resolvePlugin([
      [
        '@babel/preset-env',
        {
          modules: false,
          useBuiltIns: 'entry',
          corejs: 3,
        },
      ],
      '@babel/preset-typescript',
      '@babel/preset-react',
    ]),
    // 扩展用户自定义添加的Babel插件
    plugins: zenConfig.addBabelPlugins && Array.isArray(zenConfig.addBabelPlugins)
      ? resolvePlugin(plugins).concat(zenConfig.addBabelPlugins)
      : resolvePlugin(plugins)
  };
};
