/*
 * @Author: luzc
 * @Date:  2019/10/02
 * @Description: webpack 默认属性
 */
/* eslint-disable indent */
const OptimizeCSSAssetsPlugin = require('optimize-css-assets-webpack-plugin');
const UglifyJsPlugin = require('uglifyjs-webpack-plugin');
const getBaseConfig = require('./webpack.base.js');
const deepClone = require('lodash/cloneDeep');
const getBabelConfig = require('./getBabelConfig');

const TYPESCRIPT_LOADER = require.resolve('ts-loader');
const BABEL_LOADER = require.resolve('babel-loader');

module.exports = zenConfig => {
  const babelConfig = getBabelConfig(zenConfig);
  const baseConfig = getBaseConfig('production', zenConfig);

  baseConfig.devtool(false);

  // jsx loader 去掉exclude node_modules 因为使用第三方插件，压缩时可能报错
  baseConfig.module.rule('jsx')
  .test(/\.(jsx|js)?$/)
  .use('babel-loader')
  .loader(BABEL_LOADER)
  .options(Object.assign({}, deepClone(babelConfig), { cacheDirectory: true }));

  // tsx loader
  baseConfig.module.rule('tsx')
    .test(/\.(tsx|ts)?$/)
    .use('babel-loader')
    .loader(BABEL_LOADER)
    .options(Object.assign({}, deepClone(babelConfig), { cacheDirectory: true }))
    .end()
    .use('ts-loader')
    .loader(TYPESCRIPT_LOADER)
    .options({ transpileOnly: true });

  // uglify js file
  baseConfig.optimization
    .minimizer('UglifyJsPlugin')
    .use(UglifyJsPlugin, [{
      sourceMap: false,
      cache: true,
      parallel: true,
      uglifyOptions: {
        compress: {
          unused: false
        },
        output: {
          ascii_only: true,
          comments: 'some',
          beautify: false,
        },
        mangle: true,
      },
    }]);

  // optimize css file
  baseConfig.optimization
    .minimizer('OptimizeCSSAssetsPlugin')
    .use(OptimizeCSSAssetsPlugin, [{
      cssProcessorOptions: {
        cssDeclarationSorter: false,
        reduceIdents: false,
        parser: require('postcss-safe-parser'),
      },
    }]);

  return baseConfig;
};
