/*
 * @Author: luzc
 * @Date:  2019/10/02
 * @Description: 用户改变zend.config.js时重新启动服务
 */
const path = require('path');
const chokidar = require('chokidar');

let server = null;
let watcher = null;

module.exports = ({ onHook, log, context }) => {
  const { command, rootDir } = context;

  if (command === 'dev') {
    if (!watcher) {
      const configPath = path.resolve(rootDir, 'zend.config.js');
      watcher = chokidar.watch(configPath, {
        ignoreInitial: true,
      });

      const onUserChange = () => {
        console.log('\n');
        log.info('zend.config.js 已经被修改');
        if (!server) {
          log.error('dev server is not ready');
        } else {
          server.close();
          server = null;
          log.info('正在重启开发环境...');
          process.send({ type: 'RESTART_DEV' });
        }
      };

      watcher.on('change', () => {
        onUserChange();
      });

      watcher.on('error', (error) => {
        log.error('fail to watch file', error);
        process.exit(1);
      });
    }

    onHook('afterDevServer', (devServer) => {
      server = devServer;
    });
  }
};
