const merge = require('lodash/merge');

const deleteSlash = target => target && target.endsWith('/') && target.replace(/\/$/, '');

module.exports = ({ chainWebpack }, proxyConfig) => {
  const proxyRules = Object.entries(proxyConfig);

  const proxy = proxyRules
    .map(([match, opts]) => {
      // 是否关闭代理设置
      const { enable, target, ...proxyRule } = opts;
      if (enable !== false) {
        return merge({
          target,
          changeOrigin: true,
          logLevel: 'warn',
          secure: false,
          onProxyRes: function onProxyReq(proxyRes, req) {
            // 添加请求头部直观信息
            proxyRes.headers['x-proxy-by'] = 'zend-proxy';
            proxyRes.headers['x-proxy-match'] = match;
            proxyRes.headers['x-proxy-target'] = target;

            // 去除尾部 / 符号
            const distTarget = deleteSlash(target);
            proxyRes.headers['x-proxy-target-path'] = distTarget + req.url;
          },
          onError: function onError(err, req, res) {
            const distTarget = deleteSlash(target);
            // 不能代理的时候
            res.writeHead(500, {
              'x-proxy-by': 'zend-proxy',
              'x-proxy-match': match,
              'x-proxy-target': target,
              'x-proxy-target-path': distTarget + req.url
            });
            res.end(`proxy server error: ${err.message}`);
          },
        }, proxyRule, { context: match });
      }
      return false;
    })
    .filter((v) => v);

  chainWebpack((config) => {
    config.devServer.proxy(proxy);
  });
};
